
from odoo import api
from odoo.tests import tagged

from odoo.addons.loempia.tests.common import MockRequest, TestLoempia
from odoo.addons.loempia.controllers.frontend import LoempiaDisputesController

@tagged('post_install', '-at_install')
class TestLoempiaPurchase(TestLoempia):

    @classmethod
    def setUpClass(self):
        super(TestLoempiaPurchase, self).setUpClass()
        self.Controller = LoempiaDisputesController()
        self.registry.enter_test_mode(self.cr)

    @classmethod
    def tearDownClass(self):
        self.registry.leave_test_mode()
        super(TestLoempiaPurchase, self).tearDownClass()

    def test_claim_page(self):
        """
        This test ensures that the claim page can be accessed either by the maintainer
        either by the customer either by an apps manager. Moreover, some values in the
        rendering context are also evaluated:

        1) Ensure that template is effectively loempia.claim_page
        2) Ensure correctness of values: "purchased", "can_request_intervention",
           "can_request_refund"
        """
        # 01 Buyer
        env = api.Environment(self.env.cr, self.customer.id, {})
        with MockRequest(env, website=self.website.with_env(env)):
            response = self.Controller.claim_page("Module_A", self.purchase_A.order_name)
            self.assertEqual(response.template, 'loempia.claim_page')
            render_ctx = response.qcontext
            self.assertTrue(render_ctx.get('purchased'))
            self.assertTrue(not render_ctx.get('can_request_intervention'))
            self.assertTrue(not render_ctx.get('can_request_refund'))
            self.assertTrue(render_ctx.get('remaining_days') == 60)

        # 02 Manager
        env = api.Environment(self.env.cr, self.env.ref('base.user_admin').id, {})
        with MockRequest(env, website=self.website.with_env(env)):
            response = self.Controller.claim_page("Module_A", self.purchase_A.order_name)
            self.assertEqual(response.template, 'loempia.claim_page')
            self.assertTrue(response.qcontext.get('can_request_intervention'))  # No messages of at least 15 days, only manager should be able to create an intervention request

        # 03 Maintainer
        env = api.Environment(self.env.cr, self.maintainer.id, {})
        with MockRequest(env, website=self.website.with_env(env)):
            response = self.Controller.claim_page("Module_A", self.purchase_A.order_name)
            self.assertEqual(response.template, 'loempia.claim_page')
            self.assertTrue(response.qcontext.get('can_request_refund'))  # Maintainer have the rights to open a refund request

    def test_fuzzy_support(self):
        """
        This test ensures that the support link will only redirect you to the support
        page only and only if you bought the module.
        """
        env = api.Environment(self.env.cr, self.maintainer.id, {})
        with MockRequest(env, website=self.website.with_env(env)):
            response = self.Controller.fuzzy_support(self.module_12_A.id)
            self.assertEqual(response.template, 'loempia.claim_page_no_access')

        env = api.Environment(self.env.cr, self.customer.id, {})
        with MockRequest(env, website=self.website.with_env(env)):
            response = self.Controller.fuzzy_support(self.module_12_A.id)
            self.assertEqual(response.template, 'loempia.claim_page')

    def test_get_price(self):
        """
        This test is used to validate the price computation on purchase.
        """
        self.purchase_A.sudo().write({'quantity': 5})
        self.assertEqual(self.purchase_A.price, 50, msg="Purchase of five modules of price single of 10")

    def test_01_user_has_purchase(self):
        """
        This test ensures that we cannot have a purchase referring an entreprise module on the
        Apps store.
        """
        purchased = self.env['loempia.module.purchase'].with_user(self.customer.id).user_has_purchase(self.module_enterprise)
        self.assertEqual(False, purchased, msg="You can not buy an Odoo enterprise module on the apps store")

    def test_02_user_has_purchase(self):
        """
        This test ensure that if you bought a version, you have access to all the version
        associated to the version's package.
        """
        purchased = self.purchase_A.with_user(self.customer.id).user_has_purchase(self.module_11_A)
        self.assertEqual(True, purchased, msg="User bought version 12.0 of module which grants him access to v 11.0")

        purchased = self.purchase_A.with_user(self.customer.id).user_has_purchase(self.module_12_A)
        self.assertEqual(True, purchased, msg="User bought this version")

        purchased = self.purchase_A.with_user(self.customer.id).user_has_purchase(self.module_13_A)
        self.assertEqual(False, purchased, msg="V13.0 is a commercial version which has not be bought")

    def test_03_user_has_purchase(self):
        """
        This test ensure that you have a purchase on a Free module, only if you have
        bought all its paying dependencies.
        """
        purchased = self.env['loempia.module.purchase'].with_user(self.customer_bis.id).user_has_purchase(self.module_free)
        self.assertEqual(False, purchased, msg="Customer bis did not purchase the paying dependency of the free module")

        purchased = self.env['loempia.module.purchase'].with_user(self.customer.id).user_has_purchase(self.module_free)
        self.assertEqual(True, purchased, msg="Customer purchase the paying dependency of the free module")

    def test_claim_message(self):
        """
        This test ensure that a customer is able to contact the author through the
        support page.
        """
        env = api.Environment(self.env.cr, self.customer.id, {})
        with MockRequest(env, website=self.website.with_env(env)):
            self.Controller.post_message_claim(purchase_id=self.purchase_A.id, **{'message': 'Test'})
            self.assertEqual(len(self.purchase_A.public_message_ids), 1)
